library ieee;
use ieee.std_logic_1164.all;

package spi_bus_pkg is
    generic(
        CHIP_SELECTS : positive);

    subtype ssel_rng is natural range CHIP_SELECTS - 1 downto 0;
    subtype ssel_vst is std_logic_vector(ssel_rng);
    
    port group spi_master is
        mosi : out std_logic_vector(3 downto 0);
        miso : in std_logic_vector(3 downto 0);
        sclk : out std_logic;
        ssel : out ssel_vst;
    end;

    -- could be replaced by "alias spi_record is spi_master'record_type;"
    type spi_record is record
        mosi : std_logic_vector(3 downto 0);
        miso : std_logic_vector(3 downto 0);
        sclk : std_logic;
        ssel : ssel_vst;
    end record;
    
--    unused?
--    port group qspi_slave is
--        mosi : in std_logic_vector(3 downto 0);
--        miso : out std_logic_vector(3 downto 0);
--        sclk : in std_logic;
--        cs   : in std_logic;
--    end;
    
    port group spi_slave is
        din  : in std_logic;
        dout : out std_logic;
        sclk : in std_logic;
        cs   : in std_logic;
    end;
    
    function select_qspi_slave(master: spi_record; i : ssel_rng)
        return spi_slave;
    
    procedure spi_bits (
        group spi  : spi_slave;
        signal bits : in  positive;
        signal xmit : in  std_logic_vector;
        signal recv : out std_logic_vector;
    
        Tpd  : in  time;
        Ts   : in  time
    );

end package spi_bus_pkg;

package body spi_bus_pkg is

    function select_qspi_slave(master: spi_record; i : ssel_rng)
        return spi_slave is
    begin
        return (master.mosi(0), master.miso(0), master.sclk, master.ssel(i));
    end function select_qspi_slave;
    
    procedure spi_bits (
        group spi  : spi_slave;
        signal bits : in  positive;
        signal xmit : in  std_logic_vector;
        signal recv : out std_logic_vector;
    
        Tpd  : in  time;
        Ts   : in  time
    ) is
        variable xd : std_logic_vector(bits-1 downto 0);
        variable rd : std_logic_vector(bits-1 downto 0);
    begin
        xd := xmit;
        for i in 0 to bits-1 loop
            wait until spi.sclk = '1';
            spi.dout <= xd(bits-i-1) after Tpd;
            rd(i) := spi.din'delayed(Ts);
        end loop;
        recv <= rd;
    end procedure spi_bits;
    
end package body spi_bus_pkg;

