library ieee;
use ieee.std_logic_1164.all;
use work.spi_bus_3slv.all;

entity Testbench is
end entity Testbench;

architecture TB of Testbench is

    signal clk : std_logic;
    signal rst : std_logic;

    signal spi : spi_record;
    signal monitored_voltage, driven_voltage : real;

begin
    DUT: entity work.FPGA
        port map (
            clk => clk,
            rst => rst,
--            ...
            -- Because qspi_master is a view onto the spi_r, this
            -- mapping is allowable.
            
            spi => spi -- with implicit conversion record -> port group
--          spi => (spi.mosi, spi.miso, spi.sclk, spi.ssel) -- alternative without implicit conversion
        );

    MEMORY: entity work.qspi_flash
        port map (
            spi => select_qspi_slave(spi, 0)
        );

    ADC: entity work.spi_adc
        generic map (
            VREF => 3.0
        ) port map (
            spi => select_qspi_slave(spi, 1),
            ain => monitored_voltage
        );

    DAC: entity work.spi_dac
        generic map (
            VREF => 3.0
        ) port map (
            spi => select_qspi_slave(spi, 2),
            aout => driven_voltage
        );

    -- And our resistive pullups.
    spi.miso <= (others => 'Z');

end architecture TB;
